@@ -249,10 +249,8 @@ class Extend_Object:
             if len(decode_parms) != 0:
                 if len(filter) != len(decode_parms):
                     raise ValueError(
-                        (
-                            "filter ({}) and decode_parms ({}) must be arrays of "
-                            " same length"
-                        ).format(repr(filter), repr(decode_parms))
+                        f"filter ({repr(filter)}) and decode_parms "
+                        f"({repr(decode_parms)}) must be arrays of same length"
                     )
             if len(filter) == 1:
                 filter = filter[0]
@@ -312,8 +310,9 @@ class Extend_Pdf:
                     meta['dc:description'] = 'Put the Abstract here'
 
         Args:
-            set_pikepdf_as_editor: Update the metadata to show that this
-                version of pikepdf is the most recent software to modify the metadata.
+            set_pikepdf_as_editor: Automatically update the metadata ``pdf:Producer``
+                to show that this version of pikepdf is the most recent software to
+                modify the metadata, and ``xmp:MetadataDate`` to timestamp the update.
                 Recommended, except for testing.
 
             update_docinfo: Update the standard fields of DocumentInfo
@@ -370,14 +369,21 @@ class Extend_Pdf:
         """
         return Outline(self, max_depth=max_depth, strict=strict)
 
-    def make_stream(self, data: bytes) -> Stream:
+    def make_stream(self, data: bytes, d=None, **kwargs) -> Stream:
         """
         Create a new pikepdf.Stream object that is attached to this PDF.
 
         Args:
             data (bytes): Binary data for the stream object
+            d: Dictionary portion of the stream object
+            kwargs: Keyword arguments to define the dictionary. Do not set
+                /Filter or /Length here as pikepdf will manage these.
+
+        Example:
+            pdf.make_stream(b"Binary data here", Type=Name.XObject, Subtype=Name.Image)
+
         """
-        return Stream(self, data)
+        return Stream(self, data, d, **kwargs)
 
     def add_blank_page(
         self, *, page_size: Tuple[Numeric, Numeric] = (612.0, 792.0)
@@ -961,4 +967,4 @@ class Extend_Page:
 @augments(Token)
 class Extend_Token:
     def __repr__(self):
-        return 'pikepdf.Token({}, {})'.format(self.type_, self.raw_value)
+        return f'pikepdf.Token({self.type_}, {self.raw_value})'
